#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Class download licenses and extract data from them."""

import os
import attr
import json
import logging
from typing import List, Dict, Any
from .exceptions import UnableOpenFileData

_LOGGER = logging.getLogger(__name__)


@attr.s(slots=True)
class Licenses:
    """Class detect all licenses from downloaded data."""

    data = attr.ib(init=False, type=str)
    json_data = attr.ib(init=True, type=Dict[str, Any], default=dict())
    licenses: List[Any] = list()
    licenses_list: List[Any] = list()

    def __attrs_post_init__(self) -> None:
        """Run methods."""
        self.load_data()

    def load_data(self, file_path: str = "data/spdx_licenses.json") -> None:
        """Load data for classes variables."""
        file = os.path.join(os.path.dirname(os.path.realpath(__file__)), file_path)
        try:
            with open(file) as f:
                data = f.read()
                self.data = data
                self.json_data = json.loads(self.data)
                _LOGGER.debug("File spdx_licenses.json was successful loaded")
        except Exception:
            _LOGGER.critical("Could not open/read file: %s", file_path)
            raise UnableOpenFileData

        self._extract()

    def _extract(self) -> None:
        """Extract licenses from downloaded data."""
        try:
            for i in self.json_data["licenses"]:
                # original data
                self.licenses.append(i)
                # list of data
                li = list()
                li.append(i["name"])
                li.append(i["licenseId"])

                if i["licenseId"] != i["licenseId"].replace("-", " "):
                    # abbreviation without "-"
                    li.append(i["licenseId"].replace("-", " "))
                else:
                    li.append(i["licenseId"])

                self.licenses_list.append(li)
        except Exception as e:
            _LOGGER.warning("Something bad with Indexing: %s", e)



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Init package."""

from .solver import Solver
from typing import Dict, Any, List, Union, Optional

__title__ = "license-solver"
__version__ = "0.1.4"
__author__ = "Viliam Podhajecky <vpodhaje@redhat.com>"


def detect_license(
    input_data: Union[Dict[str, Any], str, List[str], List[Dict[str, Any]]],
    package_name: Optional[str] = None,
    package_version: Optional[str] = None,
    raise_on_error: Optional[bool] = True,
) -> Dict[str, Any]:
    """Detect license with license-solver."""
    condition = True if package_name and package_version else False

    try:
        license_solver = Solver()

        if isinstance(input_data, dict) or isinstance(input_data, str):
            license_solver.solve_from_file(input_data)
        elif isinstance(input_data, dict):
            for enter in input_data:
                license_solver.solve_from_file(enter)

        return (
            license_solver.get_output_dict(package_name=package_name, package_version=package_version)
            if condition
            else license_solver.get_output_dict()
        )

    except Exception:
        if raise_on_error:
            raise Exception

        if condition:
            return Solver.get_empty_dict()
        else:
            return {}


__all__ = [
    "__version__",
    "detect_license",
]



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""File is proposed for creating Package objects."""

import re
import yaml
import os
import logging
from typing import Union, Tuple, List, Optional, Dict
from .exceptions import UnableOpenFileData

_LOGGER = logging.getLogger(__name__)


def _detect_version_and_delete(string: str) -> Union[Tuple[str, str], Tuple[str, None]]:
    """
    Delete version of license from string.

    :returns tuple
        - license name without version - first output
        - version - second output.
    """
    regex = r"( v\d+\.| \d| version |, version)(\.\d|\d)*[\w]*"

    try:
        find = re.search(regex, string)
        if find:
            return re.sub(regex, "", string).strip(), find.group(0).strip()
        else:
            return re.sub(regex, "", string).strip(), None
    except Exception:
        return re.sub(regex, "", string).strip(), None


class Package:
    """Object which store values metadata."""

    def __init__(self) -> None:
        """Init."""
        self.name: str = ""
        self.version: str = ""
        self.license: Dict[str, str] = dict()
        self.license_version: str = ""
        self.classifier: List[List[str]] = list()
        self.file_path: str = ""

    def set_package_name(self, package_name: Optional[str]) -> None:
        """Set package name."""
        if isinstance(package_name, str):
            self.name = package_name
            _LOGGER.debug("Set name package: %s", package_name)
        else:
            _LOGGER.debug("Unsuccessful set name package")

    def set_version(self, package_version: Optional[str]) -> None:
        """Set version of package name."""
        if isinstance(package_version, str):
            self.version = package_version
            _LOGGER.debug("Set name package: %s", package_version)
        else:
            _LOGGER.debug("Unsuccessful set version package")

    def set_license(self, license_name: Tuple[List[str], bool]) -> None:
        """Set type of license."""
        file_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data", "license_without_versions.yaml")
        try:
            with open(file_path) as f:
                data = yaml.safe_load(f)
                licenses_without_version = data["license-no-versions"]
        except Exception:
            raise UnableOpenFileData

        if len(license_name[0]) > 1:
            if license_name[0][0] in licenses_without_version:
                self.license["full_name"] = license_name[0][0]
                self.license["identifier_spdx"] = license_name[0][1]
                self.license["identifier"] = license_name[0][2]
                self.set_license_version("LICENSE-WITHOUT-VERSION")
                _LOGGER.debug("Set license %s and version %s", license_name[0], "LICENSE-WITHOUT-VERSION")
            elif license_name[1]:
                _license_name_no_version, _license_version = _detect_version_and_delete(
                    license_name[0][len(license_name[0]) - 1]
                )
                self.license["full_name"] = license_name[0][0]
                self.license["identifier_spdx"] = license_name[0][1]
                self.license["identifier"] = license_name[0][2]

                if _license_version is None:
                    self.set_license_version("UNDETECTED")
                    _LOGGER.debug("Set license %s and version %s", license_name[0], "UNDETECTED")
                else:
                    self.set_license_version(_license_version)
                    _LOGGER.debug("Set license %s and version %s", license_name[0], _license_version)
            else:
                self.license["full_name"] = license_name[0][0]
                self.license["identifier_spdx"] = license_name[0][1]
                self.license["identifier"] = license_name[0][2]
                self.set_license_version("UNDETECTED")
                _LOGGER.debug("Set license %s and version %s", license_name[0], "UNDETECTED")
        elif len(license_name[0]) == 1:
            self.license["full_name"] = license_name[0][0]
            self.license["identifier_spdx"] = "UNDETECTED"
            self.license["identifier"] = "UNDETECTED"
            self.set_license_version("UNDETECTED")
            _LOGGER.debug("Set license %s and version %s", license_name[0], "UNDETECTED")
        else:
            self.license["full_name"] = "UNDETECTED"
            self.license["identifier_spdx"] = "UNDETECTED"
            self.license["identifier"] = "UNDETECTED"
            self.set_license_version("UNDETECTED")
            _LOGGER.debug("Set license %s and version %s", list(["UNDETECTED"]), "UNDETECTED")

    def set_license_version(self, license_version: str) -> None:
        """Set version of license."""
        self.license_version = license_version

    def set_classifier(self, classifier: Optional[List[str]]) -> None:
        """Set classifier."""
        if classifier is None:
            self.classifier = [["UNDETECTED"]]
            return

        if len(self.classifier) == 0:
            self.classifier = list([classifier])
        else:
            if not (classifier in self.classifier):
                self.classifier.append(classifier)

    def set_file_path(self, path: str) -> None:
        """Set file path."""
        self.file_path = path

    def print(self) -> None:
        """Print class property on STDOUT."""
        print(
            f"package:\t\t {self.name}\n"
            f"version:\t\t {self.version}\n"
            f"license:\t\t {self.license}\n"
            f"license_version: {self.license_version}\n"
            f"classifier:\t\t {self.classifier}\n"
            f"file_path:\t\t {self.file_path}"
        )



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Exception hierarchy used in the whole license-solver implementation."""


class LicenseSolverException(Exception):  # noqa: N818
    """A base license-solver exception in license-solver's exception hierarchy."""


class UnableOpenFile(LicenseSolverException):  # noqa: N818
    """An exception raised if unable to open file."""


class UnableOpenFileData(UnableOpenFile):  # noqa: N818
    """An exception raised if unable to open internal files in data/."""

    def __init__(self, txt: str = "Internal file can't be loaded"):
        """Create message."""
        super().__init__(f"{txt}")



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""A Class compare classifier and license."""

import os
import re
import yaml
import attr
import logging
from typing import List, Any, Dict
from .package import Package

_LOGGER = logging.getLogger(__name__)


def _delete_brackets(license_list: str) -> str:
    return re.sub(r"(\(?)(\)?)", "", license_list).strip()


def _delete_brackets_and_content(license_list: str) -> str:
    return re.sub(r"\(.*?\)", "", license_list).strip()


@attr.s(slots=True)
class Comparator:
    """Class Comparator compare classifiers and licenses."""

    _comparator_dictionary: Dict[str, Any] = attr.ib(init=False)

    def __attrs_post_init__(self) -> None:
        """Open dictionary for comparing license and classifier."""
        file_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data", "comparator_dictionary.yaml")
        with open(file_path) as f:
            try:
                self._comparator_dictionary = yaml.safe_load(f)
            except yaml.YAMLError:
                _LOGGER.warning("Can't open data/comparator_dictionary.yaml or broken file")
                raise yaml.YAMLError

    def cmp(self, package: Package) -> bool:
        """
        Compare License and Classifier from package data.

        :param package: Package from input
        :return: True if match, False if not
        """
        license_name = package.license
        classifier_name = package.classifier

        debug_tab = 10 * "\t"

        if not license_name or not classifier_name:
            return True

        license_list = [license_name["full_name"], license_name["identifier_spdx"], license_name["identifier"]]

        for x in classifier_name:

            _LOGGER.debug("Compare license and classifier:\n" "%s%s\n" "%s%s", debug_tab, license_name, debug_tab, x)

            if (
                list(set(license_list) & set(x))
                or self.search_in_dictionary(license_list, x)
                or classifier_name[0][0] == "UNDETECTED"
                or license_list[0] == "UNKNOWN"
                or license_list[0].lower() == "the unlicense"
            ):
                _LOGGER.debug("Found match or alias")
                return True

        _LOGGER.debug("No match")
        return False

    def search_in_dictionary(self, license_name: List[str], classifier: List[str]) -> bool:
        """
        Search for alias in data/comparator_dictionary.yaml.

        :param license_name: License to compare with classifier
        :param classifier: Classifier to compare with license
        :return: True if found match, False if not
        """
        if len(license_name) == 0:
            return False

        if (
            classifier[0] == "UNDETECTED" and self._comparator_dictionary["classifier"].get(classifier[0]) is not None
        ) or (
            classifier[0] != "UNDETECTED" and self._comparator_dictionary["classifier"].get(classifier[1]) is not None
        ):
            for x in self._comparator_dictionary["classifier"].get(classifier[1]):
                if x == license_name[0]:
                    return True

        return False



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Setup configuration for adviser module."""

import os
import sys
from setuptools import setup
from setuptools.command.test import test as TestCommand  # noqa


class Test(TestCommand):
    """Introduce test command to run testsuite using pytest."""

    _IMPLICIT_PYTEST_ARGS = [
        "--verbose",
        "-vv",
        "tests/",
    ]

    user_options = [("pytest-args=", "a", "Arguments to pass into py.test")]

    def initialize_options(self):
        """Initialize cli options."""
        super().initialize_options()
        self.pytest_args = None

    def finalize_options(self):
        """Finalize cli options."""
        super().finalize_options()
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        """Run module tests."""
        import pytest

        passed_args = list(self._IMPLICIT_PYTEST_ARGS)

        if self.pytest_args:
            self.pytest_args = [arg for arg in self.pytest_args.split() if arg]
            passed_args.extend(self.pytest_args)

        sys.exit(pytest.main(passed_args))


try:
    from setuptools import find_namespace_packages
except ImportError:
    # A dirty workaround for older setuptools.
    def find_namespace_packages(path="thoth"):
        """Find namespace packages alternative."""
        packages = set()
        for dir_name, dir_names, file_names in os.walk(path):
            if os.path.basename(dir_name) != "__pycache__":
                packages.add(dir_name.replace("/", "."))

        return sorted(packages)


def get_install_requires():
    """Get requirements for adviser module."""
    with open("requirements.txt", "r") as requirements_file:
        res = requirements_file.readlines()
        return [req.split(" ", maxsplit=1)[0] for req in res if req]


def get_version():
    """Get current version of adviser module."""
    with open(os.path.join("thoth", "license_solver", "__init__.py")) as f:
        content = f.readlines()

    for line in content:
        if line.startswith("__version__ ="):
            # dirty, remove trailing and leading chars
            return line.split(" = ")[1][1:-2]
    raise ValueError("No version identifier found")


def read(fname):
    """Read."""
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


VERSION = get_version()
setup(
    name="thoth-license-solver",
    version=VERSION,
    description="Package and package stack license-solver for the Thoth project",
    author="Viliam Podhajecky",
    author_email="vpodhaje@redhat.com",
    license="GPLv3+",
    long_description=read("README.rst"),
    packages=find_namespace_packages(),
    url="https://github.com/thoth-station/license-solver",
    package_data={
        "thoth.license_solver": [
            "data/comparator_dictionary.yaml",
            "data/license_dictionary.json",
            "data/pypi_classifiers.txt",
            "data/spdx_licenses.json",
            "data/license_without_versions.yaml",
            "py.typed",
        ]
    },
    entry_points={"console_scripts": ["thoth-license-solver=thoth.license_solver.cli:cli"]},
    zip_safe=False,
    install_requires=get_install_requires(),
    cmdclass={"test": Test},
    long_description_content_type="text/x-rst",
    command_options={
        "build_sphinx": {
            "version": ("setup.py", VERSION),
            "release": ("setup.py", VERSION),
        }
    },
)
